import java.util.Random;
import javafx.scene.input.MouseEvent;
import prog1.graphics.ApplicationBase;
import prog1.graphics.Image;
import static prog1.graphics.Graphics.*;

// Windows:
// javac -cp .;prog1javalib.jar MemoryGame.java
// java -cp .;prog1javalib.jar MemoryGame

// OS X:
// javac -cp .:prog1javalib.jar MemoryGame.java
// java -cp .:prog1javalib.jar MemoryGame

public class MemoryGame extends javafx.application.Application {

    private final static int COLUMNS = 8;
    private final static int ROWS = 8;
    private final double WIDTH = COLUMNS * 70; // window width
    private final double HEIGHT = ROWS * 70 + 25; // window height
    private final Card[][] field = new Card[ROWS][COLUMNS];
    private final Image[][] fieldImages = new Image[ROWS][COLUMNS];
    
    private final Card[] openCards = new Card[2]; // currently open cards (at most two)
    private int openCardsCount = 0;

    private int pairsFound = 0;
    private int pairsTried = 0;
    
    public MemoryGame() {
        int i = 0;
        for (int y = 0; y < ROWS; y++) {
            for (int x = 0; x < COLUMNS; x++) {
                Card card = new Card(0x2600 + i/2, Card.State.BACK);
                card.setFrontClickFunction(this::onFrontClicked, card);
                card.setBackClickFunction(this::onBackClicked, card);
                field[y][x] = card;
                i++;
            }
        }
        Card.setEmptyClickFunction(this::onFrontClicked, null);
        shuffleField();
        updateField();
    }
    
    private void onFrontClicked(MouseEvent event, Object target) {
		filpOpenCards();
        updateField();
    }
    
    private void onBackClicked(MouseEvent event, Object target) {
		filpOpenCards();
		Card card=(Card)target;
		card.setState(Card.State.FRONT);
		openCards[openCardsCount] = card;
		openCardsCount++;
		evaluateOpenCards();
        updateField();
    }
	
	private void filpOpenCards() {
		if(openCardsCount==-1){
			openCardsCount = 0;
			openCards[0].setState(Card.State.BACK);
			openCards[1].setState(Card.State.BACK);
		}
	}
    
    private void evaluateOpenCards() {
		if(openCardsCount==2){
			if(openCards[0].equals(openCards[1])){
				pairsFound++;
				openCardsCount = 0;
			}else{
				openCardsCount = -1;
			}
			pairsTried++;
		}
    }
    
    private void shuffleField() {
        Random rnd = new Random(System.currentTimeMillis());
		int ub= 1;
		int r = 0;
        for (int y = 0; y < ROWS; y++) {
            for (int x = 0; x < COLUMNS; x++) {
				r = rnd.nextInt(ub);
				int ry = r/ROWS;
				int rx = r%ROWS;
				Card card = field[x][y];
				field[x][y] = field[rx][ry];
				field[rx][ry] = card;
				ub++;
			}
		}	
    }
    
    private void updateField() {
        for (int y = 0; y < ROWS; y++) {
            for (int x = 0; x < COLUMNS; x++) {
                fieldImages[y][x] = field[y][x].getImage();
            }
        }
    }

    private Image onDraw() {
		Image top = grid(fieldImages);
		Image bottom = overlay(text(pairsFound+" pairs found in "+pairsTried+" tries", 16, "black"), rectangle(WIDTH, 25, "transparent"));
        return above(top, bottom);
    }

    @Override
    public void start(javafx.stage.Stage stage) {
        ApplicationBase.start("Memory", WIDTH, HEIGHT, stage, this::onDraw);
    }

}
